class DeepDataIntegrations(object):
    def __init__(self, client):
        self.client = client

    def create_a_connection(self, data):
        """
        Create a connection resource


        Args:
            data:

        Returns:

        """
        return self.client._post("/connections", json=data)

    def retrieve_a_connection(self, connection_id):
        """
        Retrieve an existing connection resource


        Args:
            connection_id:

        Returns:

        """
        return self.client._get("/connections/{}".format(connection_id))
    
    def update_a_connection(self, connection_id, data):
        """
        Update an existing connection resource


        Args:
            connection_id:
            data:

        Returns:

        """
        return self.client._put("/connections/{}".format(connection_id), json=data)
    
    def delete_a_connection(self, connection_id):
        """
        Delete an existing connection resource


        Args:
            connection_id:

        Returns:

        """
        return self.client._delete("/connections/{}".format(connection_id))
    
    def list_all_connections(self):
        raise NotImplementedError

    def create_an_ecommerce_customer(self, data):
        """
        Create a new e-commerce customer resource


        Args:
            data:

        Returns:

        """
        return self.client._post("/ecomCustomers", json=data)

    def retrieve_an_ecommerce_customer(self, customer_id):
        """
        Retrieve an existing e-commerce customer resource


        Args:
            customer_id:

        Returns:

        """
        return self.client._get("/ecomCustomers/{}".format(customer_id))

    def update_an_ecommerce_customer(self, customer_id, data):
        """
        Retrieve an existing e-commerce customer resource


        Args:
            customer_id:
            data:

        Returns:

        """
        return self.client._put("/ecomCustomers/{}".format(customer_id), json=data)
    
    def delete_an_ecommerce_customer(self, customer_id):
        """
        Delete an existing e-commerce customer resource


        Args:
            customer_id:

        Returns:

        """
        return self.client._delete("/ecomCustomers/{}".format(customer_id))

    def list_all_ecommerce_customer(self):
        raise NotImplementedError

    def create_an_ecommerce_order(self, data):
        """
        Create a new e-commerce order resource


        Args:
            data:

        Returns:

        """
        return self.client._post("/ecomOrders", json=data)

    def retrieve_an_ecommerce_order(self, order_id):
        """
        Retrieve an existing new e-commerce order resource


        Args:
            order_id:

        Returns:

        """
        return self.client._get("/ecomOrders/{}".format(order_id))

    def delete_an_ecommerce_order(self, order_id):
        """
        Delete an existing e-commerce order resource


        Args:
            order_id:

        Returns:

        """
        return self.client._delete("/ecomOrders/{}".format(order_id))

    def list_all_ecommerce_orders(self):
        raise NotImplementedError

    def update_an_ecommerce_order(self, order_id, data):
        """
        Update an existing ecommerce order/cart resource.


        Args:
            order_id:
            data:

        Returns:

        """
        return self.client._put("/ecomOrders/{}".format(order_id), json=data)

    def list_ecommerce_order_products(self):
        raise NotImplementedError

    def list_ecommerce_order_products_for_a_specific_ecommerce_order(self):
        raise NotImplementedError

    def retrieve_an_ecommerce_order_product(self):
        raise NotImplementedError



class Webhooks(object):
    def __init__(self, client):
        self.client = client

    def create_a_webhook(self, data):
        """
        Create a new webhook


        Args:
            data:

        Returns:

        """
        return self.client._post("/webhooks", json=data)

    def retrieve_a_webhook(self, webhook_id):
        """
        Retrieve an existing webhook


        Args:
            webhook_id:

        Returns:

        """
        return self.client._get("/webhooks/{}".format(webhook_id))

    def update_a_webhook(self, webhook_id, data):
        """
        Update an existing webhook


        Args:
            webhook_id:
            data:

        Returns:

        """
        return self.client._put("/webhooks/{}".format(webhook_id), json=data)

    def delete_a_webhook(self, webhook_id):
        """
        Delete an existing webhook


        Args:
            webhook_id:

        Returns:

        """
        return self.client._delete("/webhooks/{}".format(webhook_id))

    def list_all_webhooks(self, **params):
        """
        List all existing webhooks


        Returns:

        """
        return self.client._get("/webhooks", params=params)

    def list_all_webhook_events(self):
        """
        List all available webhook events


        Returns:

        """
        return self.client._get("/webhooks/events")



class Lists(object):
    def __init__(self, client):
        self.client = client

    def create_a_list(self, data):
        """
        A list is a group of contacts that campaigns can be sent to.
        Every campaign is required to be sent to at least one list.
        Contacts can be added to and removed from lists manually in
        the ActiveCampaign UI, via automations, or via the API.

        NOTE: When creating a new list, it is important to then associate
        that list to a group permission as seen in "Create a list group permission" below.


        Args:
            data:

        Returns:

        """
        return self.client._post("/lists", json=data)

    def retrieve_a_list(self, list_id):
        """
        Retrieve an existing list


        Args:
            list_id:

        Returns:

        """
        return self.client._get("/lists/{}".format(list_id))

    def delete_a_list(self, list_id):
        """
        Delete an existing list


        Args:
            list_id:

        Returns:

        """
        return self.client._delete("/lists/{}".format(list_id))

    def retrieve_all_lists(self, **params):
        """
        Retrieve all existing deals


        Returns:

        """
        return self.client._get("/lists", params=params)

    def create_a_list_group_permission(self, data):
        """

        Args:
            data:

        Returns:

        """
        return self.client._post("/listGroups", json=data)



class Users(object):
    def __init__(self, client):
        self.client = client

    def create_a_user(self, data):
        """
        Create a new user


        Args:
            data:

        Returns:

        """
        return self.client._post("/users", json=data)

    def retrieve_a_user(self, user_id):
        """
        Retrieve an existing user


        Args:
            user_id:

        Returns:

        """
        return self.client._get("/users/{}".format(user_id))

    def retrieve_a_user_by_email(self, email):
        """
        Retrieve an existing user by looking up their email address


        Args:
            email:

        Returns:

        """
        return self.client._get("/users/email/{}".format(email))

    def retrieve_a_user_by_username(self, username):
        """
        Retrieve an existing user by looking up their username


        Args:
            username:

        Returns:

        """
        return self.client._get("/users/username/{}".format(username))

    def retrieve_logged_in_user(self):
        """
        Retrieve the logged-in user


        Returns:

        """
        return self.client._get("/users/me")

    def update_a_user(self, user_id, data):
        """
        Update an existing user


        Args:
            user_id:
            data:

        Returns:

        """
        return self.client._put("/users/{}".format(user_id), json=data)

    def delete_a_user(self, user_id):
        """
        Delete an existing user


        Args:
            user_id:

        Returns:

        """
        return self.client._delete("/users/{}".format(user_id))

    def list_all_users(self, **params):
        """
        List all existing users


        Returns:

        """
        return self.client._get("/users", params=params)

    def create_a_group(self):
        raise NotImplementedError

    def retrieve_a_group(self):
        raise NotImplementedError

    def update_a_group(self):
        raise NotImplementedError

    def delete_a_group(self):
        raise NotImplementedError

    def list_all_groups(self):
        raise NotImplementedError



class Tags(object):
    def __init__(self, client):
        self.client = client

    def create_a_tag(self, data):
        """
        Create a new contact


        Args:
            data:

        Returns:

        """
        return self.client._post("/tags", json=data)

    def retrieve_a_tag(self, tag_id):
        """
        Retrieve an existing tag


        Args:
            tag_id:

        Returns:

        """
        return self.client._get("/tags/{}".format(tag_id))

    def update_a_tag(self, tag_id, data):
        """
        Update an existing tag


        Args:
            tag_id:
            data:

        Returns:

        """
        return self.client._put("/tags/{}".format(tag_id), json=data)

    def delete_a_tag(self, tag_id):
        """
        Delete an existing contact


        Args:
            tag_id:

        Returns:

        """
        return self.client._delete("/tags/{}".format(tag_id))

    def list_all_tags(self, **params):
        """
        List all tags

        Returns:

        """
        return self.client._get("/tags", params=params)


class Deals(object):
    def __init__(self, client):
        self.client = client

    def create_a_deal(self, data):
        """
        Create a new deal


        Args:
            data:

        Returns:

        """
        return self.client._post("/deals", json=data)

    def retrieve_a_deal(self, deal_id):
        """
        Retrieve an existing deal


        Args:
            deal_id:

        Returns:

        """
        return self.client._get("/deals/{}".format(deal_id))

    def update_a_deal(self, deal_id, data):
        """
        Update an existing deal


        Args:
            deal_id:
            data:

        Returns:

        """
        return self.client._put("/deals/{}".format(deal_id), json=data)

    def delete_a_deal(self, deal_id):
        """
        Delete an existing deal


        Args:
            deal_id:

        Returns:

        """
        return self.client._delete("/deals/{}".format(deal_id))

    def list_all_deals(self, **params):
        """
        Retrieve all existing deals


        Returns:

        """
        return self.client._get("/deals", params=params)

    def create_a_deal_note(self, deal_id, data):
        """
        Create a new note for a deal


        Args:
            deal_id:
            data:

        Returns:

        """
        return self.client._post("/deals/{}/notes".format(deal_id), json=data)

    def update_a_deal_note(self, deal_id, note_id, data):
        """
        Update an existing note for a deal


        Args:
            deal_id:
            note_id:
            data:

        Returns:

        """
        return self.client._put("/deals/{}/notes/{}".format(deal_id, note_id), json=data)

    def create_a_pipeline(self):
        raise NotImplementedError

    def retrieve_a_pipeline(self):
        raise NotImplementedError

    def update_a_pipeline(self):
        raise NotImplementedError

    def delete_a_pipeline(self):
        raise NotImplementedError

    def list_all_pipelines(self, **params):
        """
        Retrieve all existing pipelines


        Returns:

        """
        return self.client._get("/dealGroups", params=params)

    def create_a_stage(self):
        raise NotImplementedError

    def retrieve_a_stage(self):
        raise NotImplementedError

    def update_a_stage(self):
        raise NotImplementedError

    def delete_a_stage(self):
        raise NotImplementedError

    def list_all_stages(self, **params):
        """
        Retrieve all existing stages


        Returns:

        """
        return self.client._get("/dealStages", params=params)

    def move_deals_to_another_stage(self):
        raise NotImplementedError

    def create_a_custom_field(self):
        raise NotImplementedError

    def retrieve_custom_fields_by_id(self, field_id):
        """
        Retrieve all existing custom fields


        Args:
            field_id:

        Returns:

        """
        return self.client._get("/deals/{}/dealCustomFieldData".format(field_id))

    def update_a_custom_field(self):
        raise NotImplementedError

    def delete_a_custom_field(self):
        raise NotImplementedError

    def list_all_custom_fields(self, **params):
        """

        Returns:

        """
        return self.client._get("/dealCustomFieldMeta", params=params)

    def create_a_custom_field_value(self):
        raise NotImplementedError

    def bulk_create_a_custom_field_value(self):
        raise NotImplementedError

    def retrieve_a_custom_field_value(self):
        raise NotImplementedError

    def update_a_custom_field_value(self):
        raise NotImplementedError

    def bulk_update_a_custom_field_value(self):
        raise NotImplementedError

    def delete_a_custom_field_value(self):
        raise NotImplementedError

    def list_all_custom_field_values(self, **params):
        """

        Returns:

        """
        return self.client._get("/dealCustomFieldData", params=params)

    def create_a_secondary_contact(self):
        raise NotImplementedError

    def retrieve_a_secondary_contact(self):
        raise NotImplementedError

    def update_a_secondary_contact(self):
        raise NotImplementedError

    def delete_a_secondary_contact(self):
        raise NotImplementedError

    def list_all_secondary_contacts(self):
        raise NotImplementedError
